@@ -31,15 +31,20 @@ try:
     from oslo_vmware import vim_util
 except ImportError:
     api = None
-from six.moves import http_client
-from six.moves import urllib
 
+from six.moves import urllib
+import six.moves.urllib.parse as urlparse
+
+import requests
+from requests import adapters
+from requests.packages.urllib3.util import retry
 import six
 # NOTE(jokke): simplified transition to py3, behaves like py2 xrange
 from six.moves import range
 
 import glance_store
 from glance_store import capabilities
+from glance_store.common import utils
 from glance_store import exceptions
 from glance_store.i18n import _
 from glance_store.i18n import _LE
@@ -48,6 +53,7 @@ from glance_store import location
 
 LOG = logging.getLogger(__name__)
 
+CHUNKSIZE = 1024 * 64  # 64kB
 MAX_REDIRECTS = 5
 DEFAULT_STORE_IMAGE_DIR = '/openstack_glance'
 DS_URL_PREFIX = '/folder'
@@ -138,49 +144,6 @@ class _Reader(object):
         return self._size
 
 
-class _ChunkReader(_Reader):
-
-    def __init__(self, data, verifier=None, blocksize=8192):
-        self.blocksize = blocksize
-        self.current_chunk = b""
-        self.closed = False
-        super(_ChunkReader, self).__init__(data, verifier)
-
-    def read(self, size=None):
-        ret = b""
-        while size is None or size >= len(self.current_chunk):
-            ret += self.current_chunk
-            if size is not None:
-                size -= len(self.current_chunk)
-            if self.closed:
-                self.current_chunk = b""
-                break
-            self._get_chunk()
-        else:
-            ret += self.current_chunk[:size]
-            self.current_chunk = self.current_chunk[size:]
-        return ret
-
-    def _get_chunk(self):
-        if not self.closed:
-            chunk = self.data.read(self.blocksize)
-            chunk_len = len(chunk)
-            self._size += chunk_len
-            self.checksum.update(chunk)
-            if self.verifier:
-                self.verifier.update(chunk)
-            if chunk:
-                if six.PY3:
-                    size_header = ('%x\r\n' % chunk_len).encode('ascii')
-                    self.current_chunk = b''.join((size_header, chunk,
-                                                   b'\r\n'))
-                else:
-                    self.current_chunk = b'%x\r\n%s\r\n' % (chunk_len, chunk)
-            else:
-                self.current_chunk = b'0\r\n\r\n'
-                self.closed = True
-
-
 class StoreLocation(location.StoreLocation):
     """Class describing an VMware URI.
 
@@ -248,6 +211,16 @@ class StoreLocation(location.StoreLocation):
         if ds_name:
             self.datastore_name = ds_name[0]
 
+    @property
+    def https_url(self):
+        """
+        Creates a https url that can be used to upload/download data from a
+        vmware store.
+        """
+        parsed_url = urlparse.urlparse(self.get_uri())
+        new_url = parsed_url._replace(scheme='https')
+        return urlparse.urlunparse(new_url)
+
 
 class Store(glance_store.Store):
     """An implementation of the VMware datastore adapter."""
@@ -445,15 +418,13 @@ class Store(glance_store.Store):
                 are 201 Created and 200 OK.
         """
         ds = self.select_datastore(image_size)
+        image_file = _Reader(image_file, verifier)
+        headers = {}
         if image_size > 0:
-            headers = {'Content-Length': image_size}
-            image_file = _Reader(image_file, verifier)
+            headers.update({'Content-Length': image_size})
+            data = image_file
         else:
-            # NOTE (arnaud): use chunk encoding when the image is still being
-            # generated by the server (ex: stream optimized disks generated by
-            # Nova).
-            headers = {'Transfer-Encoding': 'chunked'}
-            image_file = _ChunkReader(image_file, verifier)
+            data = utils.chunkiter(image_file, CHUNKSIZE)
         loc = StoreLocation({'scheme': self.scheme,
                              'server_host': self.server_host,
                              'image_dir': self.store_image_dir,
@@ -463,13 +434,15 @@ class Store(glance_store.Store):
         # NOTE(arnaud): use a decorator when the config is not tied to self
         cookie = self._build_vim_cookie_header(True)
         headers = dict(headers)
-        headers['Cookie'] = cookie
-        conn_class = self._get_http_conn_class()
-        conn = conn_class(loc.server_host)
-        url = urllib.parse.quote('%s?%s' % (loc.path, loc.query))
+        headers.update({'Cookie': cookie})
+        session = new_session(self.api_insecure)
+
+        url = loc.https_url
         try:
-            conn.request('PUT', url, image_file, headers)
+            response = session.put(url, data=data, headers=headers)
         except IOError as e:
+            # TODO(sigmavirus24): Figure out what the new exception type would
+            # be in requests.
             # When a session is not authenticated, the socket is closed by
             # the server after sending the response. http_client has an open
             # issue with https that raises Broken Pipe
@@ -482,17 +455,19 @@ class Store(glance_store.Store):
                                               'url': url,
                                               'e': e}
             LOG.error(msg)
+            raise exceptions.BackendException(msg)
         except Exception:
             with excutils.save_and_reraise_exception():
                 LOG.exception(_LE('Failed to upload content of image '
                                   '%(image)s'), {'image': image_id})
-        res = conn.getresponse()
-        if res.status == http_client.CONFLICT:
+
+        res = response.raw
+        if res.status == requests.codes.conflict:
             raise exceptions.Duplicate(_("Image file %(image_id)s already "
                                          "exists!") %
                                        {'image_id': image_id})
 
-        if res.status not in (http_client.CREATED, http_client.OK):
+        if res.status not in (requests.codes.created, requests.codes.ok):
             msg = (_LE('Failed to upload content of image %(image)s. '
                        'The request returned an unexpected status: %(status)s.'
                        '\nThe response body:\n%(body)s') %
@@ -534,7 +509,15 @@ class Store(glance_store.Store):
         :param location: `glance_store.location.Location` object, supplied
                         from glance_store.location.get_location_from_uri()
         """
-        return self._query(location, 'HEAD')[2]
+        conn = None
+        try:
+            conn, resp, size = self._query(location, 'HEAD')
+            return size
+        finally:
+            # NOTE(sabari): Close the connection as the request was made with
+            # stream=True.
+            if conn is not None:
+                conn.close()
 
     @capabilities.check
     def delete(self, location, context=None):
@@ -566,30 +549,59 @@ class Store(glance_store.Store):
                 LOG.exception(_LE('Failed to delete image %(image)s '
                                   'content.') % {'image': location.image_id})
 
-    def _query(self, location, method, depth=0):
-        if depth > MAX_REDIRECTS:
+    def _query(self, location, method):
+        session = new_session(self.api_insecure)
+        loc = location.store_location
+        redirects_followed = 0
+        # TODO(sabari): The redirect logic was added to handle cases when the
+        # backend redirects http url's to https. But the store never makes a
+        # http request and hence this can be safely removed.
+        while redirects_followed < MAX_REDIRECTS:
+            conn, resp = self._retry_request(session, method, location)
+
+            # NOTE(sigmavirus24): _retry_request handles 4xx and 5xx errors so
+            # if the response is not a redirect, we can return early.
+            if not conn.is_redirect:
+                break
+
+            redirects_followed += 1
+
+            location_header = conn.headers.get('location')
+            if location_header:
+                if resp.status not in (301, 302):
+                    reason = (_("The HTTP URL %(path)s attempted to redirect "
+                                "with an invalid %(status)s status code.")
+                              % {'path': loc.path, 'status': resp.status})
+                    LOG.info(reason)
+                    raise exceptions.BadStoreUri(message=reason)
+                conn.close()
+                location = self._new_location(location, location_header)
+        else:
+            # NOTE(sigmavirus24): We exceeded the maximum number of redirects
             msg = ("The HTTP URL exceeded %(max_redirects)s maximum "
                    "redirects.", {'max_redirects': MAX_REDIRECTS})
             LOG.debug(msg)
             raise exceptions.MaxRedirectsExceeded(redirects=MAX_REDIRECTS)
+
+        content_length = int(resp.getheader('content-length', 0))
+
+        return (conn, resp, content_length)
+
+    def _retry_request(self, session, method, location):
         loc = location.store_location
         # NOTE(arnaud): use a decorator when the config is not tied to self
         for i in range(self.api_retry_count + 1):
             cookie = self._build_vim_cookie_header()
             headers = {'Cookie': cookie}
-            try:
-                conn = self._get_http_conn(method, loc, headers)
-                resp = conn.getresponse()
-            except Exception:
-                with excutils.save_and_reraise_exception():
-                    LOG.exception(_LE('Failed to access image %(image)s '
-                                      'content.') % {'image':
-                                                     location.image_id})
+            conn = session.request(method, loc.https_url, headers=headers,
+                                   stream=True)
+            resp = conn.raw
+
             if resp.status >= 400:
-                if resp.status == http_client.UNAUTHORIZED:
+                if resp.status == requests.codes.unauthorized:
                     self.reset_session()
                     continue
-                if resp.status == http_client.NOT_FOUND:
+                if resp.status == requests.codes.not_found:
                     reason = _('VMware datastore could not find image at URI.')
                     LOG.info(reason)
                     raise exceptions.NotFound(message=reason)
@@ -598,34 +610,36 @@ class Store(glance_store.Store):
                 LOG.debug(msg)
                 raise exceptions.BadStoreUri(msg)
             break
-        location_header = resp.getheader('location')
-        if location_header:
-            if resp.status not in (301, 302):
-                reason = (_("The HTTP URL %(path)s attempted to redirect "
-                            "with an invalid %(status)s status code.")
-                          % {'path': loc.path, 'status': resp.status})
-                LOG.info(reason)
-                raise exceptions.BadStoreUri(message=reason)
-            location_class = glance_store.location.Location
-            new_loc = location_class(location.store_name,
-                                     location.store_location.__class__,
-                                     uri=location_header,
-                                     image_id=location.image_id,
-                                     store_specs=location.store_specs)
-            return self._query(new_loc, method, depth + 1)
-        content_length = int(resp.getheader('content-length', 0))
+        return conn, resp
 
-        return (conn, resp, content_length)
+    def _new_location(self, old_location, url):
+        store_name = old_location.store_name
+        store_class = old_location.store_location.__class__
+        image_id = old_location.image_id
+        store_specs = old_location.store_specs
+        # Note(sabari): The redirect url will have a scheme 'http(s)', but the
+        # store only accepts url with scheme 'vsphere'. Thus, replacing with
+        # store's scheme.
+        parsed_url = urlparse.urlparse(url)
+        new_url = parsed_url._replace(scheme='vsphere')
+        vsphere_url = urlparse.urlunparse(new_url)
+        return glance_store.location.Location(store_name,
+                                              store_class,
+                                              self.conf,
+                                              uri=vsphere_url,
+                                              image_id=image_id,
+                                              store_specs=store_specs)
 
-    def _get_http_conn(self, method, loc, headers, content=None):
-        conn_class = self._get_http_conn_class()
-        conn = conn_class(loc.server_host)
-        url = urllib.parse.quote('%s?%s' % (loc.path, loc.query))
-        conn.request(method, url, content, headers)
 
-        return conn
-
-    def _get_http_conn_class(self):
-        if self.api_insecure:
-            return http_client.HTTPConnection
-        return http_client.HTTPSConnection
+def new_session(insecure=False, total_retries=None):
+    session = requests.Session()
+    if total_retries is not None:
+        http_adapter = adapters.HTTPAdapter(
+            max_retries=retry.Retry(total=total_retries))
+        https_adapter = adapters.HTTPAdapter(
+            max_retries=retry.Retry(total=total_retries))
+        session.mount('http://', http_adapter)
+        session.mount('https://', https_adapter)
+    if insecure:
+        session.verify = False
+    return session
