@@ -22,6 +22,7 @@ import mock
 import tempfile
 import uuid
 
+from keystoneclient import exceptions as ks_exceptions
 from oslo_config import cfg
 from oslo_utils import encodeutils
 from oslo_utils import units
@@ -35,6 +36,7 @@ from six.moves import range
 import swiftclient
 
 from glance_store._drivers.swift import store as swift
+from glance_store._drivers.swift import utils as sutils
 from glance_store import backend
 from glance_store import BackendException
 from glance_store import capabilities
@@ -233,6 +235,12 @@ def stub_out_swiftclient(stubs, swift_store_auth_version):
 
 class SwiftTests(object):
 
+    def mock_keystone_client(self):
+        # mock keystone client functions to avoid dependency errors
+        swift.ks_v3 = mock.MagicMock()
+        swift.ks_session = mock.MagicMock()
+        swift.ks_client = mock.MagicMock()
+
     @property
     def swift_store_user(self):
         return 'tenant:user1'
@@ -285,10 +293,13 @@ class SwiftTests(object):
         resp_full = b''.join([chunk for chunk in image_swift.wrapped])
         resp_half = resp_full[:len(resp_full) // 2]
         resp_half = six.BytesIO(resp_half)
+        manager = swift.get_manager_for_store(self.store, loc.store_location,
+                                              ctxt)
+
         image_swift.wrapped = swift.swift_retry_iter(resp_half, image_size,
                                                      self.store,
                                                      loc.store_location,
-                                                     ctxt)
+                                                     manager)
         self.assertEqual(image_size, 5120)
 
         expected_data = b"*" * FIVE_KB
@@ -337,6 +348,7 @@ class SwiftTests(object):
     def test_add(self):
         """Test that we can add an image via the swift backend."""
         moves.reload_module(swift)
+        self.mock_keystone_client()
         self.store = Store(self.conf)
         self.store.configure()
         expected_swift_size = FIVE_KB
@@ -374,6 +386,7 @@ class SwiftTests(object):
         conf['default_swift_reference'] = 'store_2'
         self.config(**conf)
         moves.reload_module(swift)
+        self.mock_keystone_client()
         self.store = Store(self.conf)
         self.store.configure()
 
@@ -424,6 +437,7 @@ class SwiftTests(object):
             conf['default_swift_reference'] = variation
             self.config(**conf)
             moves.reload_module(swift)
+            self.mock_keystone_client()
             self.store = Store(self.conf)
             self.store.configure()
             loc, size, checksum, _ = self.store.add(image_id, image_swift,
@@ -454,6 +468,7 @@ class SwiftTests(object):
         conf['swift_store_container'] = 'noexist'
         self.config(**conf)
         moves.reload_module(swift)
+        self.mock_keystone_client()
 
         self.store = Store(self.conf)
         self.store.configure()
@@ -500,6 +515,7 @@ class SwiftTests(object):
         conf['swift_store_container'] = 'noexist'
         self.config(**conf)
         moves.reload_module(swift)
+        self.mock_keystone_client()
         self.store = Store(self.conf)
         self.store.configure()
         loc, size, checksum, _ = self.store.add(expected_image_id,
@@ -545,6 +561,8 @@ class SwiftTests(object):
         conf['swift_store_multiple_containers_seed'] = 2
         self.config(**conf)
         moves.reload_module(swift)
+        self.mock_keystone_client()
+
         self.store = Store(self.conf)
         self.store.configure()
         loc, size, checksum, _ = self.store.add(expected_image_id,
@@ -579,6 +597,7 @@ class SwiftTests(object):
         conf['swift_store_multiple_containers_seed'] = 2
         self.config(**conf)
         moves.reload_module(swift)
+        self.mock_keystone_client()
 
         expected_image_id = str(uuid.uuid4())
         expected_container = 'randomname_' + expected_image_id[:2]
@@ -652,6 +671,44 @@ class SwiftTests(object):
                  mock.call(b'')]
         verifier.update.assert_has_calls(calls)
 
+    @mock.patch('glance_store._drivers.swift.utils'
+                '.is_multiple_swift_store_accounts_enabled',
+                mock.Mock(return_value=True))
+    def test_add_with_verifier_small(self):
+        """Test that the verifier is updated for smaller images."""
+        swift_size = FIVE_KB
+        base_byte = b"12345678"
+        swift_contents = base_byte * (swift_size // 8)
+        image_id = str(uuid.uuid4())
+        image_swift = six.BytesIO(swift_contents)
+
+        self.store = Store(self.conf)
+        self.store.configure()
+        orig_max_size = self.store.large_object_size
+        orig_temp_size = self.store.large_object_chunk_size
+        custom_size = 6 * units.Ki
+        verifier = mock.MagicMock(name='mock_verifier')
+
+        try:
+            self.store.large_object_size = custom_size
+            self.store.large_object_chunk_size = custom_size
+            self.store.add(image_id, image_swift, swift_size,
+                           verifier=verifier)
+        finally:
+            self.store.large_object_chunk_size = orig_temp_size
+            self.store.large_object_size = orig_max_size
+
+        # Confirm verifier update called expected number of times
+        self.assertEqual(verifier.update.call_count, 2)
+
+        # define one chunk of the contents
+        swift_contents_piece = base_byte * (swift_size // 8)
+
+        # confirm all expected calls to update have occurred
+        calls = [mock.call(swift_contents_piece),
+                 mock.call(b'')]
+        verifier.update.assert_has_calls(calls)
+
     @mock.patch('glance_store._drivers.swift.utils'
                 '.is_multiple_swift_store_accounts_enabled',
                 mock.Mock(return_value=False))
@@ -857,6 +914,7 @@ class SwiftTests(object):
         conf = copy.deepcopy(SWIFT_CONF)
         self.config(**conf)
         moves.reload_module(swift)
+        self.mock_keystone_client()
         self.store = Store(self.conf)
         self.store.configure()
 
@@ -896,6 +954,7 @@ class SwiftTests(object):
         conf = copy.deepcopy(SWIFT_CONF)
         self.config(**conf)
         moves.reload_module(swift)
+        self.mock_keystone_client()
         self.store = Store(self.conf)
         self.store.configure()
 
@@ -922,7 +981,7 @@ class SwiftTests(object):
         loc = location.get_location_from_uri(uri, conf=self.conf)
         self.store.delete(loc)
 
-        self.assertRaises(exceptions.NotFound, self.store.get, loc)
+        self.assertRaises(ks_exceptions.NotFound, self.store.get, loc)
 
     def test_delete_non_existing(self):
         """
@@ -1040,6 +1099,112 @@ class SwiftTests(object):
         self.assertEqual(container_headers['X-Container-Write'],
                          'frank:*,jim:*')
 
+    @mock.patch("glance_store._drivers.swift."
+                "connection_manager.MultiTenantConnectionManager")
+    def test_get_connection_manager_multi_tenant(self, manager_class):
+        manager = mock.MagicMock()
+        manager_class.return_value = manager
+        self.config(swift_store_multi_tenant=True)
+        store = Store(self.conf)
+        store.configure()
+        loc = mock.MagicMock()
+        swift.get_manager_for_store(store, loc)
+        self.assertEqual(swift.get_manager_for_store(store, loc),
+                         manager)
+
+    @mock.patch("glance_store._drivers.swift."
+                "connection_manager.SingleTenantConnectionManager")
+    def test_get_connection_manager_single_tenant(self, manager_class):
+        manager = mock.MagicMock()
+        manager_class.return_value = manager
+        store = Store(self.conf)
+        store.configure()
+        loc = mock.MagicMock()
+        swift.get_manager_for_store(store, loc)
+        self.assertEqual(swift.get_manager_for_store(store, loc),
+                         manager)
+
+    def test_get_connection_manager_failed(self):
+        store = mock.MagicMock()
+        loc = mock.MagicMock()
+        self.assertRaises(NotImplementedError, swift.get_manager_for_store,
+                          store, loc)
+
+    @mock.patch("glance_store._drivers.swift.store.ks_v3")
+    @mock.patch("glance_store._drivers.swift.store.ks_session")
+    @mock.patch("glance_store._drivers.swift.store.ks_client")
+    def test_init_client_multi_tenant(self,
+                                      mock_client, mock_session, mock_v3):
+        """Test that keystone client was initialized correctly"""
+        # initialize store and connection parameters
+        self.config(swift_store_multi_tenant=True)
+        store = Store(self.conf)
+        store.configure()
+        ref_params = sutils.SwiftParams(self.conf).params
+        default_ref = self.conf.glance_store.default_swift_reference
+        default_swift_reference = ref_params.get(default_ref)
+        # prepare client and session
+        trustee_session = mock.MagicMock()
+        trustor_session = mock.MagicMock()
+        main_session = mock.MagicMock()
+        trustee_client = mock.MagicMock()
+        trustee_client.session.get_user_id.return_value = 'fake_user'
+        trustor_client = mock.MagicMock()
+        trustor_client.session.auth.get_auth_ref.return_value = {
+            'roles': [{'name': 'fake_role'}]
+        }
+        trustor_client.trusts.create.return_value = mock.MagicMock(
+            id='fake_trust')
+        main_client = mock.MagicMock()
+        mock_session.Session.side_effect = [trustor_session, trustee_session,
+                                            main_session]
+        mock_client.Client.side_effect = [trustor_client, trustee_client,
+                                          main_client]
+        # initialize client
+        ctxt = mock.MagicMock()
+        client = store.init_client(location=mock.MagicMock(), context=ctxt)
+        # test trustor usage
+        mock_v3.Token.assert_called_once_with(
+            auth_url=default_swift_reference.get('auth_address'),
+            token=ctxt.auth_token,
+            project_id=ctxt.tenant
+        )
+        mock_session.Session.assert_any_call(auth=mock_v3.Token())
+        mock_client.Client.assert_any_call(session=trustor_session)
+        # test trustee usage and trust creation
+        tenant_name, user = default_swift_reference.get('user').split(':')
+        mock_v3.Password.assert_any_call(
+            auth_url=default_swift_reference.get('auth_address'),
+            username=user,
+            password=default_swift_reference.get('key'),
+            project_name=tenant_name,
+            user_domain_id=default_swift_reference.get('user_domain_id'),
+            user_domain_name=default_swift_reference.get('user_domain_name'),
+            project_domain_id=default_swift_reference.get('project_domain_id'),
+            project_domain_name=default_swift_reference.get(
+                'project_domain_name')
+        )
+        mock_session.Session.assert_any_call(auth=mock_v3.Password())
+        mock_client.Client.assert_any_call(session=trustee_session)
+        trustor_client.trusts.create.assert_called_once_with(
+            trustee_user='fake_user', trustor_user=ctxt.user,
+            project=ctxt.tenant, impersonation=True,
+            role_names=['fake_role']
+        )
+        mock_v3.Password.assert_any_call(
+            auth_url=default_swift_reference.get('auth_address'),
+            username=user,
+            password=default_swift_reference.get('key'),
+            trust_id='fake_trust',
+            user_domain_id=default_swift_reference.get('user_domain_id'),
+            user_domain_name=default_swift_reference.get('user_domain_name'),
+            project_domain_id=default_swift_reference.get('project_domain_id'),
+            project_domain_name=default_swift_reference.get(
+                'project_domain_name')
+        )
+        mock_client.Client.assert_any_call(session=main_session)
+        self.assertEqual(main_client, client)
+
 
 class TestStoreAuthV1(base.StoreBaseTest, SwiftTests,
                       test_store_capabilities.TestStoreCapabilitiesChecking):
@@ -1064,6 +1229,7 @@ class TestStoreAuthV1(base.StoreBaseTest, SwiftTests,
         moxfixture = self.useFixture(moxstubout.MoxStubout())
         self.stubs = moxfixture.stubs
         stub_out_swiftclient(self.stubs, conf['swift_store_auth_version'])
+        self.mock_keystone_client()
         self.store = Store(self.conf)
         self.config(**conf)
         self.store.configure()
@@ -1102,6 +1268,33 @@ class TestStoreAuthV3(TestStoreAuthV1):
         conf['swift_store_user'] = 'tenant:user1'
         return conf
 
+    @mock.patch("glance_store._drivers.swift.store.ks_v3")
+    @mock.patch("glance_store._drivers.swift.store.ks_session")
+    @mock.patch("glance_store._drivers.swift.store.ks_client")
+    def test_init_client_single_tenant(self,
+                                       mock_client, mock_session, mock_v3):
+        """Test that keystone client was initialized correctly"""
+        # initialize client
+        store = Store(self.conf)
+        store.configure()
+        uri = "swift://%s:key@auth_address/glance/%s" % (
+            self.swift_store_user, FAKE_UUID)
+        loc = location.get_location_from_uri(uri, conf=self.conf)
+        ctxt = mock.MagicMock()
+        store.init_client(location=loc.store_location, context=ctxt)
+        # check that keystone was initialized correctly
+        tenant = None if store.auth_version == '1' else "tenant"
+        username = "tenant:user1" if store.auth_version == '1' else "user1"
+        mock_v3.Password.assert_called_once_with(
+            auth_url=loc.store_location.swift_url + '/',
+            username=username, password="key",
+            project_name=tenant,
+            project_domain_id=None, project_domain_name=None,
+            user_domain_id=None, user_domain_name=None,)
+        mock_session.Session.assert_called_once_with(auth=mock_v3.Password())
+        mock_client.Client.assert_called_once_with(
+            session=mock_session.Session())
+
 
 class FakeConnection(object):
     def __init__(self, authurl=None, user=None, key=None, retries=5,
@@ -1327,7 +1520,7 @@ class TestMultiTenantStoreConnections(base.StoreBaseTest):
         self.location = swift.StoreLocation(specs, self.conf)
         self.addCleanup(self.conf.reset)
 
-    def test_basic_connection(self):
+    def test_basic_connection_no_catalog(self):
         self.store.configure()
         connection = self.store.get_connection(self.location,
                                                context=self.context)
@@ -1340,6 +1533,59 @@ class TestMultiTenantStoreConnections(base.StoreBaseTest):
         self.assertEqual(connection.preauthtoken, '0123')
         self.assertEqual(connection.os_options, {})
 
+    def test_connection_with_endpoint_from_catalog(self):
+        self.store.configure()
+        self.context.service_catalog = [
+            {
+                'endpoint_links': [],
+                'endpoints': [
+                    {
+                        'region': 'RegionOne',
+                        'publicURL': 'https://scexample.com',
+                    },
+                ],
+                'type': 'object-store',
+                'name': 'Object Storage Service',
+            }
+        ]
+        connection = self.store.get_connection(self.location,
+                                               context=self.context)
+        self.assertIsNone(connection.authurl)
+        self.assertEqual(connection.auth_version, '1')
+        self.assertIsNone(connection.user)
+        self.assertIsNone(connection.tenant_name)
+        self.assertIsNone(connection.key)
+        self.assertEqual(connection.preauthurl, 'https://scexample.com')
+        self.assertEqual(connection.preauthtoken, '0123')
+        self.assertEqual(connection.os_options, {})
+
+    def test_connection_with_no_endpoint_found(self):
+        self.store.configure()
+        self.context.service_catalog = [
+            {
+                'endpoint_links': [],
+                'endpoints': [
+                    {
+                        'region': 'RegionOne',
+                        'publicURL': 'https://scexample.com',
+                    },
+                ],
+                'type': 'object-store',
+                'name': 'Object Storage Service',
+            }
+        ]
+        self.store.service_type = 'incorrect-store'
+        connection = self.store.get_connection(self.location,
+                                               context=self.context)
+        self.assertIsNone(connection.authurl)
+        self.assertEqual(connection.auth_version, '1')
+        self.assertIsNone(connection.user)
+        self.assertIsNone(connection.tenant_name)
+        self.assertIsNone(connection.key)
+        self.assertEqual(connection.preauthurl, 'https://example.com')
+        self.assertEqual(connection.preauthtoken, '0123')
+        self.assertEqual(connection.os_options, {})
+
 
 class TestMultiTenantStoreContext(base.StoreBaseTest):
 
